/*
 Author : MEZIERE Fabien
 Project : SRO2004
 Website : https://www.instructables.com/member/-SRO2-/
 Description : This is the source code of my WineCabinet, for more information or any suggestion go to the website ;)

   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>. 1
*/

#include <Event.h>
#include <Timer.h>

#include <OneWire.h>
#include <DallasTemperature.h>
#include <AFMotor.h>

#define ONE_WIRE_BUS A0
#define POT_TEMPERATURE A1
#define MODULE_PELTIER A2
#define LED_ROUGE A3
#define LED_VERTE A4

#define HYSTERESIS 1 // Definit un hysteresis de 1 degré pour le declenchement/arret du refroidissement

// Declaration des variables
int temperatureMesuree = 0;
int temperatureReglee = 0;

Timer t;
int timerEvent;

void stopVentilateurPeltier();

// Configuration du capteur de température
OneWire oneWire(ONE_WIRE_BUS); // Bus One Wire sur la pin A0 de l'arduino
DallasTemperature capteurTemperature(&oneWire); // Utilisation du bus Onewire pour le capteur de temperature
DeviceAddress sensorDeviceAddress; // Vérifie la compatibilité des capteurs avec la librairie

// Création des objets pour le controle des moteurs
AF_DCMotor ventilateurPeltier(1, MOTOR12_64KHZ); // Créer le moteur #1, PWM à 64KHz
AF_DCMotor ventilateurBrassage(2, MOTOR12_64KHZ); // Créer le moteur #2, PWM à 64KHz


void setup(void) {

  // Configuration des pin en sorties
  pinMode(MODULE_PELTIER, OUTPUT);
  pinMode(LED_ROUGE, OUTPUT);
  pinMode(LED_VERTE, OUTPUT);

  // Allume la LED de statut en verte
  digitalWrite(LED_ROUGE, HIGH);
  digitalWrite(LED_VERTE, LOW);
  // Configure la vitesse des ventilateurs au maximum
  ventilateurPeltier.setSpeed(255); // Fixe la vitesse du ventilateur du peltier au maximum
  ventilateurBrassage.setSpeed(255); // Fixe la vitesse du ventilateur de brassage au maximum
  // Configure le capteur de température
  capteurTemperature.begin(); //Activation des capteurs
  capteurTemperature.getAddress(sensorDeviceAddress, 0); //Demande l'adresse du capteur à l'index 0 du bus
  capteurTemperature.setResolution(sensorDeviceAddress, 12); //Résolutions possibles: 9,10,11,12

  // Mise en route du ventilateur de brassage
  ventilateurBrassage.run(FORWARD);
  Serial.println("Démarre le ventilateur de brassage d'air");

  // Initialisation de la connexion serie pour le debug
  Serial.begin(9600);
}


void loop(void) {
  // Lecture de la température reglée avec le potentiomètre (le 7 correspond à la température qui est de 7 minimum à 20 maximum)
  temperatureReglee = (analogRead(POT_TEMPERATURE) / 78) + 10;
  Serial.print("Temperature reglee :");
  Serial.println(temperatureReglee);

  // Lecture de la sonde de température
  capteurTemperature.requestTemperatures();
  temperatureMesuree = capteurTemperature.getTempCByIndex(0);
  Serial.print("Temperature mesuree :");
  Serial.println(temperatureMesuree);

  if (temperatureMesuree > (temperatureReglee + HYSTERESIS))
  {
    t.stop(timerEvent); // Arrete le timer des ventilateurs peltier au cas où il serait encore actif
    Serial.println("Lancement du refroidissement");

    digitalWrite(MODULE_PELTIER, HIGH); // Mise en route du module peltier
    ventilateurPeltier.run(FORWARD);  // Mise en route des ventilateurs peltier
    // Allume la LED de statut en rouge
    digitalWrite(LED_VERTE, HIGH);
    digitalWrite(LED_ROUGE, LOW);
    do
    {
      // Lecture de la sonde de température
      capteurTemperature.requestTemperatures();
      temperatureMesuree = capteurTemperature.getTempCByIndex(0);
      // Lecture de la température reglée avec le potentiomètre
      temperatureReglee = (analogRead(POT_TEMPERATURE) / 78) + 10;

      Serial.print("Temperature reglee :");
      Serial.println(temperatureReglee);
      Serial.print("Temperature mesuree :");
      Serial.println(temperatureMesuree);
    } while (temperatureMesuree > temperatureReglee - HYSTERESIS); // Attendre tant que la temperature mesuree est superieur a la temperature reglee

    Serial.println("Arret du refroidissement");

    // Allume la LED de statut en verte
    digitalWrite(LED_ROUGE, HIGH);
    digitalWrite(LED_VERTE, LOW);
    // Arret du module peltier
    digitalWrite(MODULE_PELTIER, LOW);
    timerEvent = t.after(180000, stopVentilateurPeltier);
  }
  t.update();
  delay(10);
}

void stopVentilateurPeltier()
{
  ventilateurPeltier.run(RELEASE);
  t.stop(timerEvent);
}
